﻿' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports System.Security
Imports System.Security.Cryptography
Imports System.Runtime.InteropServices
Imports System.IO
Imports Microsoft.Win32

Public Class Misc

    Private Shared sizeUnits() As String = {"Bytes", "KB", "MB", "GB", "TB", "PB", "EB"}


    ' Show an error message as a debug message (now messagebox)
    Public Shared Sub ShowDebugError(ByVal ex As Exception)
        Debug.WriteLine("================================================================")
        Debug.WriteLine("GOT AN ERROR")
        Debug.WriteLine("       Source : " & ex.Source)
        Debug.WriteLine("       Message : " & ex.Message)
        Debug.WriteLine("       Trace : " & ex.StackTrace)
        Debug.WriteLine("================================================================")

        ' Beep for debug mode :-)
#If RELEASE_MODE = 0 Then
            Beep()
#End If
    End Sub

    ' Escape will close the form frm
    Public Shared Sub CloseWithEchapKey(ByRef frm As System.Windows.Forms.Form)
        frm.KeyPreview = True
        Dim oo As New System.Windows.Forms.KeyEventHandler(AddressOf handlerCloseForm_)
        AddHandler frm.KeyDown, oo
    End Sub

    ' Open a file/URL
    Public Shared Function ShellOpenFile(ByVal filePath As String, ByVal handle As IntPtr) As Boolean
        Dim SEI As Native.Api.NativeStructs.ShellExecuteInfo = Nothing
        With SEI
            .fMask = Native.Api.NativeEnums.ShellExecuteInfoMask.SEE_MASK_NOCLOSEPROCESS Or _
                    Native.Api.NativeEnums.ShellExecuteInfoMask.SEE_MASK_INVOKEIDLIST Or _
                    Native.Api.NativeEnums.ShellExecuteInfoMask.SEE_MASK_FLAG_NO_UI
            .cbSize = System.Runtime.InteropServices.Marshal.SizeOf(SEI)
            .hwnd = handle
            .lpVerb = vbNullChar
            .lpFile = filePath
            .lpParameters = vbNullChar
            .lpDirectory = vbNullChar
            .nShow = 0
            .hInstApp = IntPtr.Zero
            .lpIDList = IntPtr.Zero
        End With

        Return Native.Api.NativeFunctions.ShellExecuteEx(SEI)
    End Function

    ' Get a formated value as a string (in Bytes, KB, MB or GB) from an Integer
    Public Shared Function GetFormatedSize(ByVal size As Integer, Optional ByVal digits As Integer = 3) As String
        Return GetFormatedSize(New Decimal(size), digits)
    End Function
    Public Shared Function GetFormatedSize(ByVal size As ULong, Optional ByVal digits As Integer = 3) As String
        Return GetFormatedSize(New Decimal(size), digits)
    End Function
    Public Shared Function GetFormatedSize(ByVal size As UInteger, Optional ByVal digits As Integer = 3) As String
        Return GetFormatedSize(New Decimal(size), digits)
    End Function
    Public Shared Function GetFormatedSize(ByVal size As Decimal, Optional ByVal digits As Integer = 3) As String
        Dim t As Decimal = size
        Dim dep As Integer = 0

        While t >= 1024
            t /= 1024
            dep += 1
        End While

        Dim d As Double = Math.Round(t, digits)

        If d > 0 Then
            Return d.ToString & " " & sizeUnits(dep)
        Else
            Return ""
        End If

    End Function

    ' Reduce working set size
    Public Shared Sub ReduceWorkingSetSize()

        Dim _hHandle As IntPtr = Native.Api.NativeFunctions.OpenProcess(Native.Api.NativeConstants.Process_SetQuota, _
                                                                        False, _
                                                                        Process.GetCurrentProcess.Id)
        If _hHandle <> IntPtr.Zero Then
            Dim _ret As Boolean = Native.Api.NativeFunctions.SetProcessWorkingSetSize(_hHandle, _
                                                                                      Native.Api.NativeConstants.InvalidHandleValue, _
                                                                                      Native.Api.NativeConstants.InvalidHandleValue)
            Native.Api.NativeFunctions.CloseHandle(_hHandle)
        End If
    End Sub

    ' Return a date/time from a number of seconds
    Public Shared Function FormatedSeconds(ByVal seconds As Long) As String
        Dim bAddComma As Boolean
        Dim Result As String = Nothing
        Dim sTemp As String
        Dim days As Long
        Dim hours As Long
        Dim minutes As Long

        If seconds <= 0 Then
            Return "0 seconds"
        End If

        seconds = CLng(Math.Floor(seconds))

        days = seconds \ 86400
        hours = (seconds - (days * 86400)) \ 3600
        minutes = (seconds - (hours * 3600) - (days * 86400)) \ 60
        seconds = seconds - (minutes * 60) - (hours * 3600) - (days * 86400)

        If seconds > 0 Then
            Result = seconds & " second" & AutoS(seconds)
        End If
        If minutes > 0 Then
            bAddComma = (Result <> Nothing)
            sTemp = minutes & " minute" & AutoS(minutes)
            If bAddComma Then
                sTemp = sTemp & ", "
            End If
            Result = sTemp & Result
        End If
        If hours > 0 Then
            bAddComma = (Result <> Nothing)
            sTemp = hours & " hour" & AutoS(hours)
            If bAddComma Then
                sTemp = sTemp & ", "
            End If
            Result = sTemp & Result
        End If
        If days > 0 Then
            bAddComma = (Result <> Nothing)
            sTemp = days & " day" & AutoS(days)
            If bAddComma Then
                sTemp = sTemp & ", "
            End If
            Result = sTemp & Result
        End If

        Return Result
    End Function
    Private Shared Function AutoS(ByVal Number As Long) As String
        If Number = 1 Then
            AutoS = ""
        Else
            AutoS = "s"
        End If
    End Function

    ' Start (or not) with windows startup
    Public Shared Sub StartWithWindows(ByVal value As Boolean)
        Try
            Dim regKey As RegistryKey
            regKey = Registry.CurrentUser.OpenSubKey("Software\Microsoft\Windows\CurrentVersion\Run", True)

            If value Then
                regKey.SetValue(Application.ProductName, Application.ExecutablePath)
            Else
                regKey.DeleteValue(Application.ProductName)
            End If
        Catch ex As Exception
            Misc.ShowDebugError(ex)
        End Try
    End Sub

    ' Set tool tips
    Public Shared Sub SetToolTip(ByVal ctrl As Control, ByVal text As String)
        Dim tToolTip As ToolTip = New System.Windows.Forms.ToolTip
        With tToolTip
            .SetToolTip(ctrl, text)
            .IsBalloon = False
            .Active = True
        End With
    End Sub

    ' Download web page
    Private Shared Function DownloadPage(ByVal sURL As String) As String

        Try
            Dim objWebRequest As System.Net.WebRequest = System.Net.HttpWebRequest.Create(sURL)
            Dim objWebResponse As System.Net.WebResponse = objWebRequest.GetResponse()
            Dim objStreamReader As System.IO.StreamReader = Nothing
            Dim ret As String = Nothing

            Try
                objStreamReader = New System.IO.StreamReader(objWebResponse.GetResponseStream())
                ret = objStreamReader.ReadToEnd()
            Finally
                If Not objWebResponse Is Nothing Then
                    objWebResponse.Close()
                End If
            End Try

            Return ret

        Catch ex As Exception
            Return Nothing
        End Try

    End Function

    ' Add a message to log
    Public Shared Sub AddToLog(ByVal action As Scenario.EAction, ByVal machine As String, ByVal message As String)
        Dim it As New ListViewItem(Date.Now.ToLongTimeString & " - " & Date.Now.ToShortDateString)
        it.SubItems.Add(action.ToString)
        it.SubItems.Add(machine)
        it.SubItems.Add(message)
        _frmMain.Invoke(New frmMain.AddToLog(AddressOf _frmMain.impAddToLog), it)
    End Sub

    ' Retrieve a string from a secure string
    Public Shared Function SecureStringToString(ByVal value As SecureString) As String
        Dim bstr As IntPtr = Marshal.SecureStringToBSTR(value)
        Try
            Return Marshal.PtrToStringBSTR(bstr)
        Finally
            Marshal.FreeBSTR(bstr)
        End Try
    End Function

    ' Encrypt a SecureString to a string
    Public Shared Function Encrypt(ByVal value As SecureString) As String
        If value Is Nothing Then
            Throw New ArgumentNullException("value")
        End If

        Dim ptr As IntPtr = Marshal.SecureStringToCoTaskMemUnicode(value)
        Try
            Dim buffer As Char() = New Char(value.Length - 1) {}
            Marshal.Copy(ptr, buffer, 0, value.Length)

            Dim data As Byte() = System.Text.Encoding.Unicode.GetBytes(buffer)
            Dim encrypted As Byte() = ProtectedData.Protect(data, Nothing, DataProtectionScope.CurrentUser)

            'return as base64 string
            Return Convert.ToBase64String(encrypted)
        Finally
            Marshal.ZeroFreeCoTaskMemUnicode(ptr)
        End Try
    End Function

    ' Decrypt a string to a SecureString
    Public Shared Function DecryptSecure(ByVal cipher As String) As SecureString
        If cipher Is Nothing Then
            Throw New ArgumentNullException("cipher")
        End If

        'parse base64 string
        Dim data As Byte() = Convert.FromBase64String(cipher)

        'decrypt data
        Dim decrypted As Byte() = ProtectedData.Unprotect(data, Nothing, DataProtectionScope.CurrentUser)

        Dim ss As New SecureString()

        'parse characters one by one - doesn't change the fact that
        'we have them in memory however...
        Dim count As Integer = System.Text.Encoding.Unicode.GetCharCount(decrypted)
        Dim bc As Integer = CInt(decrypted.Length / count)
        For i As Integer = 0 To count - 1
            ss.AppendChar(System.Text.Encoding.Unicode.GetChars(decrypted, i * bc, bc)(0))
        Next

        'mark as read-only
        ss.MakeReadOnly()
        Return ss
    End Function

    ' ========================================
    ' Private functions
    ' ========================================

    Private Shared Sub handlerCloseForm_(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)
        If e.KeyCode = System.Windows.Forms.Keys.Escape Then
            Try
                Dim _tmp As System.Windows.Forms.Form = DirectCast(sender, System.Windows.Forms.Form)
                _tmp.DialogResult = Windows.Forms.DialogResult.Cancel
                _tmp.Close()
            Catch ex As Exception
                Misc.ShowDebugError(ex)
            End Try
            e.Handled = True
        End If
    End Sub

End Class
