﻿' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports System.Xml
Imports System.Xml.Serialization
Imports System.Text
Imports System.IO

Public Class Scenarios

    ' ========================================
    ' Private
    ' ========================================

    ' List of scenarios
    Private _col As New Collections.Generic.List(Of Scenario)

    ' Semaphore to protect the list
    Private Shared _semCollection As New System.Threading.Mutex



    ' ========================================
    ' Public
    ' ========================================

    ' Return the list of scenarios
    Public ReadOnly Property GetScenarios() As Collections.Generic.List(Of Scenario)
        Get
            Return _col
        End Get
    End Property

    ' Return the semaphore
    Public Shared ReadOnly Property SemProtectScenarios() As System.Threading.Mutex
        Get
            Return _semCollection
        End Get
    End Property

    ' Add a scenario
    Public Sub Add(ByRef item As Scenario)
        _semCollection.WaitOne()
        If _col.Contains(item) = False Then
            _col.Add(item)
        End If
        _semCollection.ReleaseMutex()
    End Sub

    ' Remove a scenario
    Public Sub Remove(ByRef item As Scenario)
        _semCollection.WaitOne()
        If _col.Contains(item) Then
            _col.Remove(item)
        End If
        _semCollection.ReleaseMutex()
    End Sub

#Region "Serializarion"
    ' Save scenarios to XML
    Public Sub SaveData(ByVal file As String)
        Dim serialXML As Xml.Serialization.XmlSerializer = Nothing
        Dim streamIO As StreamWriter = Nothing
        Try
            serialXML = New Xml.Serialization.XmlSerializer(GetType(Collections.Generic.List(Of Scenario)))
            streamIO = New StreamWriter(file)
            serialXML.Serialize(streamIO, _col)
        Catch ex As Exception
            Throw ex
        Finally
            If streamIO IsNot Nothing Then
                streamIO.Close()
            End If
        End Try
    End Sub

    ' Load scenarios from XML
    Public Sub LoadData(ByVal file As String)
        Dim streamIO As StreamReader = Nothing
        Dim serialXML As Xml.Serialization.XmlSerializer = Nothing
        Try
            If System.IO.File.Exists(file) = True Then
                serialXML = New Xml.Serialization.XmlSerializer(GetType(Collections.Generic.List(Of Scenario)))
                streamIO = New StreamReader(file)
                _col = CType(serialXML.Deserialize(streamIO), Collections.Generic.List(Of Scenario))
            End If
        Catch ex As Exception
            Throw ex
        Finally
            If streamIO IsNot Nothing Then
                streamIO.Close()
            End If
        End Try
    End Sub
#End Region

End Class