﻿' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports System.Runtime.InteropServices
Imports Native.Api
Imports System.Text

Namespace Native.Objects

    Public Class Token

        ' ========================================
        ' Private constants
        ' ========================================


        ' ========================================
        ' Private attributes
        ' ========================================


        ' ========================================
        ' Public properties
        ' ========================================


        ' ========================================
        ' Other public
        ' ========================================
        Public Shared PrivilegeNames As String() = { _
                        "SeCreateTokenPrivilege", _
                        "SeAssignPrimaryTokenPrivilege", _
                        "SeLockMemoryPrivilege", _
                        "SeIncreaseQuotaPrivilege", _
                        "SeUnsolicitedInputPrivilege", _
                        "SeMachineAccountPrivilege", _
                        "SeTcbPrivilege", _
                        "SeSecurityPrivilege", _
                        "SeTakeOwnershipPrivilege", _
                        "SeLoadDriverPrivilege", _
                        "SeSystemProfilePrivilege", _
                        "SeSystemtimePrivilege", _
                        "SeProfileSingleProcessPrivilege", _
                        "SeIncreaseBasePriorityPrivilege", _
                        "SeCreatePagefilePrivilege", _
                        "SeCreatePermanentPrivilege", _
                        "SeBackupPrivilege", _
                        "SeRestorePrivilege", _
                        "SeShutdownPrivilege", _
                        "SeDebugPrivilege", _
                        "SeAuditPrivilege", _
                        "SeSystemEnvironmentPrivilege", _
                        "SeChangeNotifyPrivilege", _
                        "SeRemoteShutdownPrivilege", _
                        "SeCreateGlobalPrivilege", _
                        "SeCreateTokenPrivilege", _
                        "SeAssignPrimaryTokenPrivilege", _
                        "SeLockMemoryPrivilege", _
                        "SeIncreaseQuotaPrivilege", _
                        "SeUnsolicitedInputPrivilege", _
                        "SeMachineAccountPrivilege", _
                        "SeTcbPrivilege", _
                        "SeSecurityPrivilege", _
                        "SeTakeOwnershipPrivilege", _
                        "SeLoadDriverPrivilege", _
                        "SeSystemProfilePrivilege", _
                        "SeSystemtimePrivilege", _
                        "SeProfileSingleProcessPrivilege", _
                        "SeIncreaseBasePriorityPrivilege", _
                        "SeCreatePagefilePrivilege", _
                        "SeCreatePermanentPrivilege", _
                        "SeBackupPrivilege", _
                        "SeRestorePrivilege", _
                        "SeShutdownPrivilege", _
                        "SeDebugPrivilege", _
                        "SeAuditPrivilege", _
                        "SeSystemEnvironmentPrivilege", _
                        "SeChangeNotifyPrivilege", _
                        "SeRemoteShutdownPrivilege", _
                        "SeCreateGlobalPrivilege", _
                        "SeUndockPrivilege", _
                        "SeManageVolumePrivilege", _
                        "SeImpersonatePrivilege", _
                        "SeEnableDelegationPrivilege", _
                        "SeSyncAgentPrivilege", _
                        "SeTrustedCredManAccessPrivilege", _
                        "SeRelabelPrivilege", _
                        "SeIncreaseWorkingSetPrivilege", _
                        "SeTimeZonePrivilege", _
                        "SeCreateSymbolicLinkPrivilege"}


        ' ========================================
        ' Public functions
        ' ========================================

        ' Return elevation type of a process
        Public Shared Function GetProcessElevationTypeByTokenHandle(ByVal hTok As IntPtr, _
                                               ByRef elevation As Native.Api.NativeEnums.ElevationType) As Boolean
            If hTok.IsNotNull Then

                Dim value As Integer
                Dim ret As Integer

                ' Get tokeninfo length
                Native.Api.NativeFunctions.GetTokenInformation(hTok, _
                            NativeConstants.Token_TokenElevationType, _
                            IntPtr.Zero, _
                            0, _
                            ret)
                Using memAlloc As New Memory.MemoryAlloc(ret)
                    ' Get token information
                    Native.Api.NativeFunctions.GetTokenInformation(hTok, _
                                NativeConstants.Token_TokenElevationType, _
                                memAlloc, _
                                memAlloc.Size, _
                                ret)
                    ' Get a valid structure
                    value = memAlloc.ReadInt32(0)
                End Using
                elevation = CType(value, Native.Api.NativeEnums.ElevationType)
                Return True
            Else
                elevation = Native.Api.NativeEnums.ElevationType.Default
                Return False
            End If
        End Function

        ' Query privilege status
        Public Shared Function SetPrivilegeStatusByProcessId(ByVal processId As Integer, _
                                            ByVal seName As String, _
                                            ByVal seStatus As NativeEnums.SePrivilegeAttributes) As Boolean

            Dim hProcess As IntPtr
            Dim fRet As Boolean
            Dim lngToken As IntPtr
            Dim typLUID As NativeStructs.Luid
            Dim typTokenPriv As New NativeStructs.TokenPrivileges
            Dim newTokenPriv As IntPtr
            Dim ret2 As IntPtr

            ' Open handle to process
            hProcess = Native.Api.NativeFunctions.OpenProcess(NativeConstants.Process_QueryInformation, _
                                                            False, _
                                                            processId)

            If hProcess.IsNotNull Then
                ' Get token handle
                NativeFunctions.OpenProcessToken(hProcess, NativeConstants.Token_Query Or NativeConstants.Token_AdjustPrivileges, lngToken)

                If lngToken.IsNotNull Then

                    ' Retrieve Luid from PrivilegeName
                    If NativeFunctions.LookupPrivilegeValue(Nothing, seName, typLUID) Then

                        ' Adjust privilege
                        typTokenPriv.PrivilegeCount = 1
                        ReDim typTokenPriv.Privileges(0)
                        typTokenPriv.Privileges(0) = New NativeStructs.LuidAndAttributes
                        typTokenPriv.Privileges(0).Attributes = seStatus
                        typTokenPriv.Privileges(0).pLuid = typLUID
                        fRet = NativeFunctions.AdjustTokenPrivileges(lngToken, False, typTokenPriv, _
                                                         0, newTokenPriv, ret2)
                    End If
                    NativeFunctions.CloseHandle(lngToken)
                End If
                NativeFunctions.CloseHandle(hProcess)
            End If

            Return fRet
        End Function


        ' ========================================
        ' Private functions
        ' ========================================

    End Class

End Namespace
