﻿' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports Microsoft.Win32
Imports System.IO

Public Class frmMain

    ' Delegates for update
    Public Delegate Sub NewUpdateAvailableNotification(ByVal release As cUpdate.NewReleaseInfos)
    Public Delegate Sub NoNewUpdateAvailableNotification()
    Public Delegate Sub FailedToCheckUpDateNotification(ByVal msg As String)

    Private _ableToClose As Boolean = False
    Public WriteOnly Property AbleToClose() As Boolean
        Set(ByVal value As Boolean)
            _ableToClose = value
        End Set
    End Property

    ' Close form (hide if necessary)
    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        e.Cancel = Not (_ableToClose)
        Call hideForm()
    End Sub

    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Initialize form
        If My.Settings.StartMinimized Then
            Call hideForm()
        End If

        ' GUI
        Me.Icon = My.Resources.shutdown
        Native.Api.NativeFunctions.SetWindowTheme(Me.lvScenarios.Handle, "explorer", Nothing)
        Native.Api.NativeFunctions.SetWindowTheme(Me.lvLog.Handle, "explorer", Nothing)

        ' Fill comboboxes
        Me.chkIRemote_CheckedChanged(Nothing, Nothing)
        Me.chkRemote_CheckedChanged(Nothing, Nothing)

        ' Settings
        Me.chkActivate.Checked = My.Settings.StartEnabled
        Me.dtCountdown.Value = New Date(Date.Now.Year, 1, 1, 0, 15, 0)
        If My.Settings.DateTimeFormat.Length > 0 Then
            Me.dt.CustomFormat = My.Settings.DateTimeFormat
        Else
            Me.dt.CustomFormat = "ddd, dd MMM yyyy HH':'mm':'ss"
        End If

        ' Tooltips
        Call Misc.SetToolTip(Me.lvScenarios, "List of saved scenarios. Check (or not) a scenario to enable or disable it")
        Call Misc.SetToolTip(Me.cbAction, "Action to schedule")
        Call Misc.SetToolTip(Me.cbFrequency, "Frequency of action")
        Call Misc.SetToolTip(Me.cbImmediateAction, "Frequency of action")
        Call Misc.SetToolTip(Me.cmdAddScenario, "Add the specified scenario and save it")
        Call Misc.SetToolTip(Me.cmdBrowse, "Select the program to launch")
        Call Misc.SetToolTip(Me.cmdIBrowse, "Select the program to launch")
        Call Misc.SetToolTip(Me.cmdIGo, "Launch immediate action now")
        Call Misc.SetToolTip(Me.chkActivate, "Enable or disable scenarios")
        Call Misc.SetToolTip(Me.chkForce, "Force action")
        Call Misc.SetToolTip(Me.chkIForce, "Force action")
        Call Misc.SetToolTip(Me.chkIRemote, "Immediate action for a remote computer")
        Call Misc.SetToolTip(Me.chkAutoRemoveScenario, "Automatically remove scenario from saved list after it is launched, if it is a 'one time' kind of scenario")
        Call Misc.SetToolTip(Me.chkRemote, "Schedule action for a remote computer")
        Call Misc.SetToolTip(Me.txtIMachineName, "Remote machine name")
        Call Misc.SetToolTip(Me.txtIPath, "Path of the program to launch")
        Call Misc.SetToolTip(Me.txtIPwd, "Password to use")
        Call Misc.SetToolTip(Me.txtIUserName, "User name")
        Call Misc.SetToolTip(Me.txtMachine, "Remote machine name")
        Call Misc.SetToolTip(Me.txtPath, "Path of the program to launch")
        Call Misc.SetToolTip(Me.txtPwd, "Password to use")
        Call Misc.SetToolTip(Me.txtUser, "User name")
        Call Misc.SetToolTip(Me.dt, "Date/time of the schedule action")
        Call Misc.SetToolTip(Me.lvLog, "List of errors")
        Call Misc.SetToolTip(Me.optCountdown, "Specify a countdown")
        Call Misc.SetToolTip(Me.optDateTime, "Specify a date/time")
        Call Misc.SetToolTip(Me.dtCountdown, "Time before scenario will be launched")

        ' Load scenarios
        If System.IO.File.Exists(XML_FILE) Then
            ' Load XML file
            Call _scenarioList.LoadData(XML_FILE)
        End If
        For Each scenar As Scenario In _scenarioList.GetScenarios
            Call addScenarToLV(scenar)
        Next

        ' Initialize notify icon
        With Me.NotifyIcon
            .Icon = My.Resources.shutdown
            .Visible = True
            .ContextMenu = Me.TheContextMenu
        End With

        ' Check is SSS is up to date
        If My.Settings.AutoUpdate Then
            Program.Updater.CheckUpdates(True)
        End If

    End Sub

    Private Sub MenuItemExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemExit.Click
        Program.ExitSSS()
    End Sub

    Private Sub NotifyIcon_MouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles NotifyIcon.MouseDoubleClick
        Me.showForm()
    End Sub

    Private Sub chkIRemote_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkIRemote.CheckedChanged
        Me.gpRemote.Enabled = chkIRemote.Checked
        If Me.chkIRemote.Checked = False Then
            Me.cbImmediateAction.Items.Clear()
            Dim _items() As String = {"Restart", "Shutdown", "Poweroff", "Sleep", "Logoff", "Lock", "Start program"}
            Me.cbImmediateAction.Items.AddRange(_items)
        Else
            Me.cbImmediateAction.Items.Clear()
            Dim _items() As String = {"Restart", "Shutdown", "Poweroff", "Logoff", "Start program"}
            Me.cbImmediateAction.Items.AddRange(_items)
        End If
        Me.cmdIGo.Enabled = (Me.cbImmediateAction.Text <> Nothing)
    End Sub

    Private Sub lnklblSF_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs)
        Misc.ShellOpenFile("http://sourceforge.net/projects/sshutdown/", Me.Handle)
    End Sub

    Private Sub cbImmediateAction_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbImmediateAction.SelectedIndexChanged
        Me.txtIPath.Enabled = (Me.cbImmediateAction.Text = "Start program")
        Me.cmdIBrowse.Enabled = Me.txtIPath.Enabled
        Me.cmdIGo.Enabled = (Me.cbImmediateAction.Text <> Nothing)
    End Sub

    Private Sub Timer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer.Tick
        Me.checkScenarios()
    End Sub

    Private Sub chkActivate_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkActivate.CheckedChanged
        Me.permuteEnableState()
    End Sub

    Private Sub chkRemote_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRemote.CheckedChanged
        Me.gpRemote2.Enabled = chkRemote.Checked
        If Me.chkRemote.Checked = False Then
            Me.cbAction.Items.Clear()
            Dim _items() As String = {"Restart", "Shutdown", "Poweroff", "Sleep", "Logoff", "Lock", "Start program"}
            Me.cbAction.Items.AddRange(_items)
        Else
            Me.cbAction.Items.Clear()
            Dim _items() As String = {"Restart", "Shutdown", "Poweroff", "Logoff", "Start program"}
            Me.cbAction.Items.AddRange(_items)
        End If
        Me.cmdAddScenario.Enabled = ((Me.cbAction.Text.Length > 0) And ((Me.cbFrequency.Text.Length > 0) Or Me.optCountdown.Checked))
    End Sub

    Private Sub cbAction_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbAction.SelectedIndexChanged
        Me.txtPath.Enabled = (Me.cbAction.Text = "Start program")
        Me.cmdBrowse.Enabled = Me.txtPath.Enabled
        Me.cmdAddScenario.Enabled = ((Me.cbAction.Text.Length > 0) And ((Me.cbFrequency.Text.Length > 0) Or Me.optCountdown.Checked))
    End Sub

    Private Sub cbFrequency_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbFrequency.SelectedIndexChanged
        Me.cmdAddScenario.Enabled = ((Me.cbAction.Text.Length > 0) And ((Me.cbFrequency.Text.Length > 0) Or Me.optCountdown.Checked))
    End Sub

    Private Sub MenuItemShow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemShow.Click
        Me.showForm()
    End Sub

    ' Show the main form
    Private Sub showForm()
        Me.CenterToScreen()
        Me.WindowState = FormWindowState.Normal
        Me.ShowInTaskbar = True
        Me.Visible = True
        Me.Show()
    End Sub

    ' Hide the main form
    Private Sub hideForm()
        Me.Left = -20000
        Me.ShowInTaskbar = False
        Me.Hide()
        Call Misc.ReduceWorkingSetSize()
    End Sub

    Private Sub MenuItemHide_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHide.Click
        Me.hideForm()
    End Sub

    Private Sub cmdIBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdIBrowse.Click
        With OpenFileDialog
            .AddExtension = True
            .AutoUpgradeEnabled = True
            .CheckFileExists = True
            .CheckPathExists = True
            .Filter = "All Files (*.*)|*.*|Executables (*.exe)|*.exe"
            .Title = "Select the program to be launched"
            If .ShowDialog = Windows.Forms.DialogResult.OK Then
                Me.txtIPath.Text = .FileName
            End If
        End With
    End Sub

    Private Sub cmdBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdBrowse.Click
        With OpenFileDialog
            .AddExtension = True
            .AutoUpgradeEnabled = True
            .CheckFileExists = True
            .CheckPathExists = True
            .Filter = "All Files (*.*)|*.*|Executables (*.exe)|*.exe"
            .Title = "Select the program to be launched"
            If .ShowDialog = Windows.Forms.DialogResult.OK Then
                Me.txtPath.Text = .FileName
            End If
        End With
    End Sub

    Private Sub cmdAddScenario_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAddScenario.Click
        Dim _dat As New Scenario.TDate
        Dim _action As New Scenario.EAction
        Dim _remote As New Scenario.TRemote
        Dim _path As String
        Dim _typ As Scenario.EDate

        Select Case Me.cbFrequency.Text
            Case "One time"
                _typ = Scenario.EDate.OneTime
            Case "Daily"
                _typ = Scenario.EDate.Daily
            Case "Weekly"
                _typ = Scenario.EDate.Weekly
            Case "Monthly"
                _typ = Scenario.EDate.Monthly
        End Select

        If Me.optDateTime.Checked Then
            _dat = New Scenario.TDate(_typ, New Scenario.TTime(dt.Value))
        Else
            Dim ts As New TimeSpan(Me.dtCountdown.Value.Hour, _
                                   Me.dtCountdown.Value.Minute, _
                                   Me.dtCountdown.Value.Second)
            Dim diffDat As Date = Date.Now.Add(ts)
            _dat = New Scenario.TDate(Scenario.EDate.OneTime, _
                                      New Scenario.TTime(diffDat))
        End If

        Select Case Me.cbAction.Text
            Case "Restart"
                _action = Scenario.EAction.Restart
            Case "Shutdown"
                _action = Scenario.EAction.Shutdown
            Case "Poweroff"
                _action = Scenario.EAction.PowerOff
            Case "Sleep"
                _action = Scenario.EAction.Sleep
            Case "Logoff"
                _action = Scenario.EAction.LogOff
            Case "Lock"
                _action = Scenario.EAction.Lock
            Case "Start program"
                _action = Scenario.EAction.StartProgram
        End Select
        _remote = New Scenario.TRemote(Me.txtMachine.Text, Me.txtUser.Text, Me.txtPwd.SecureText)
        _path = Me.txtPath.Text

        Dim _t As Scenario

        If Me.chkRemote.Checked Then
            _t = New Scenario(_dat, Me.chkForce.Checked, _action, _remote, _path)
        Else
            _t = New Scenario(_dat, Me.chkForce.Checked, _action, _path)
        End If
        _t.Enabled = True
        _t.AutoRemove = Me.chkAutoRemoveScenario.Checked

        ' Add to list
        _scenarioList.Add(_t)

        _scenarioList.SaveData(XML_FILE)
        Me.addScenarToLV(_t)
    End Sub

    Friend Sub addScenarToLV(ByRef scenar As Scenario)
        Dim it As ListViewItem = Me.lvScenarios.Items.Add(scenar.Action.ToString)
        it.SubItems.Add(scenar.MachineName)
        it.SubItems.Add(scenar.TheDate.ToString)
        it.Checked = scenar.Enabled
        it.Tag = scenar
    End Sub

    Private Sub lvScenarios_ItemCheck(ByVal sender As Object, ByVal e As System.Windows.Forms.ItemCheckEventArgs) Handles lvScenarios.ItemCheck
        Dim scenar As Scenario = DirectCast(Me.lvScenarios.Items(e.Index).Tag, Scenario)
        If scenar IsNot Nothing Then
            Dim enabled As Boolean = CBool(e.NewValue)
            scenar.Enabled = enabled
        End If
    End Sub

    Private Sub lvScenarios_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles lvScenarios.KeyDown
        If e.KeyCode = Keys.Delete Then
            If Me.lvScenarios.SelectedItems.Count = 1 Then
                If MessageBox.Show("Are you sure you want to delete this scenario ?", "Delete scenario", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then
                    Dim _scenar As Scenario = DirectCast(Me.lvScenarios.Items(0).Tag, Scenario)
                    If _scenar IsNot Nothing Then
                        _scenarioList.Remove(_scenar)
                        Me.lvScenarios.Items.Remove(Me.lvScenarios.Items(0))
                    End If
                End If
            ElseIf Me.lvScenarios.SelectedItems.Count > 1 Then
                If MessageBox.Show("Are you sure you want to delete these scenarios ?", "Delete scenarios", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then
                    For i As Integer = Me.lvScenarios.SelectedItems.Count - 1 To 0 Step -1
                        Dim _scenar As Scenario = DirectCast(Me.lvScenarios.Items(i).Tag, Scenario)
                        If _scenar IsNot Nothing Then
                            _scenarioList.Remove(_scenar)
                            Me.lvScenarios.Items.Remove(Me.lvScenarios.Items(i))
                        End If
                    Next
                End If
            End If
            Me.lvScenarios.Items.Clear()
            For Each scenar As Scenario In _scenarioList.GetScenarios
                Me.addScenarToLV(scenar)
            Next
        End If
    End Sub

    Private Sub permuteEnableState()
        If Me.chkActivate.Checked = False Then
            Me.lblState.Text = "Scenarios are not enabled"
            Me.NotifyIcon.Text = Me.lblState.Text
        End If
        Me.Timer.Enabled = Me.chkActivate.Checked
        If Me.chkActivate.Checked Then
            Me.MenuItemEnableScenarios.Text = "Disable scenarios"
        Else
            Me.MenuItemEnableScenarios.Text = "Enable scenarios"
        End If
    End Sub

    Private Sub MenuItemEnableScenarios_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEnableScenarios.Click
        Me.chkActivate.Checked = Not (Me.chkActivate.Checked)
        Call permuteEnableState()
    End Sub

    Private Sub cmdDonate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Misc.ShellOpenFile("https://sourceforge.net/project/project_donations.php?group_id=268426", Me.Handle)
    End Sub

    Public Delegate Sub AddToLog(ByVal it As ListViewItem)
    Public Sub impAddToLog(ByVal it As ListViewItem)
        _frmMain.lvLog.Items.Add(it)
    End Sub

    Private Sub lvLog_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles lvLog.KeyDown
        If e.KeyCode = Keys.Delete Then
            For i As Integer = Me.lvLog.SelectedItems.Count - 1 To 0 Step -1
                Me.lvLog.Items(i).Remove()
            Next
        End If
    End Sub

    Private Sub optDateTime_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optDateTime.CheckedChanged
        Me.gpCountDown.Visible = Me.optCountdown.Checked
        Me.gpDate.Visible = Not (Me.gpCountDown.Visible)
        Me.cmdAddScenario.Enabled = ((Me.cbAction.Text.Length > 0) And ((Me.cbFrequency.Text.Length > 0) Or Me.optCountdown.Checked))
    End Sub

    Private Sub optCountdown_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optCountdown.CheckedChanged
        Me.gpCountDown.Visible = Me.optCountdown.Checked
        Me.gpDate.Visible = Not (Me.gpCountDown.Visible)
        Me.cmdAddScenario.Enabled = ((Me.cbAction.Text.Length > 0) And ((Me.cbFrequency.Text.Length > 0) Or Me.optCountdown.Checked))
    End Sub

    Private Sub cmdGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdIGo.Click
        Dim _action As Scenario.EAction

        Select Case Me.cbImmediateAction.Text
            Case "Restart"
                _action = Scenario.EAction.Restart
            Case "Shutdown"
                _action = Scenario.EAction.Shutdown
            Case "Poweroff"
                _action = Scenario.EAction.PowerOff
            Case "Sleep"
                _action = Scenario.EAction.Sleep
            Case "Logoff"
                _action = Scenario.EAction.LogOff
            Case "Lock"
                _action = Scenario.EAction.Lock
            Case "Start program"
                _action = Scenario.EAction.StartProgram
        End Select

        Me.launchImmediateScenario(_action, Me.chkIForce.Checked, Me.chkIRemote.Checked, Me.txtIPath.Text)
    End Sub

    Private Sub MenuItemIRestart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemIRestart.Click
        Call launchImmediateScenario(Scenario.EAction.Restart, True)
    End Sub

    Private Sub MenuItemIShutdown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemIShutdown.Click
        Call launchImmediateScenario(Scenario.EAction.Shutdown, True)
    End Sub

    Private Sub MenuItemIPoweroff_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemIPoweroff.Click
        Call launchImmediateScenario(Scenario.EAction.PowerOff, True)
    End Sub

    Private Sub MenuItemILock_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemILock.Click
        Call launchImmediateScenario(Scenario.EAction.Lock, True)
    End Sub

    Private Sub MenuItemILogoff_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemILogoff.Click
        Call launchImmediateScenario(Scenario.EAction.LogOff, True)
    End Sub

    Private Sub MenuItemISleep_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemISleep.Click
        Call launchImmediateScenario(Scenario.EAction.Sleep, True)
    End Sub

    ' Refresh listview
    Public Delegate Sub UpdateListOfScenarios()
    Friend Sub impUpdateListOfScenarios()
        Scenarios.SemProtectScenarios.WaitOne()
        lvScenarios.Items.Clear()
        Dim _tmp As Collections.Generic.List(Of Scenario) = _scenarioList.GetScenarios
        For Each scenar As Scenario In _tmp
            Call addScenarToLV(scenar)
        Next
        Scenarios.SemProtectScenarios.ReleaseMutex()
    End Sub



    ' ========================================
    ' Private functions
    ' ========================================

    ' Launch scenarios if necessary
    Private Sub checkScenarios()
        Dim diff As Long = Long.MaxValue
        Dim _tmp As Long
        Dim nextScenar As Scenario = Nothing

        Scenarios.SemProtectScenarios.WaitOne()
        Dim _now As New Scenario.TTime(Date.Now)
        For Each _t As Scenario In _scenarioList.GetScenarios

            ' Start scenario if necessary
            Select Case _t.TheDate.Type

                Case Scenario.EDate.Daily
                    If _t.TheDate.Time.ReadyForDaily(New Scenario.TTime(Date.Now)) Then
                        _t.Go()
                    End If

                    If _t.Enabled AndAlso _t.Done = False Then
                        _tmp = _t.TheDate.Time.DailyDiff(New Scenario.TTime(Date.Now))
                        If _tmp < diff AndAlso _tmp > 0 Then
                            diff = _tmp
                            nextScenar = _t
                        End If
                    End If

                Case Scenario.EDate.Monthly
                    If _t.TheDate.Time.ReadyForMothly(New Scenario.TTime(Date.Now)) Then
                        _t.Go()
                    End If

                    If _t.Enabled AndAlso _t.Done = False Then
                        _tmp = _t.TheDate.Time.MonthlyDiff(New Scenario.TTime(Date.Now))
                        If _tmp < diff AndAlso _tmp > 0 Then
                            diff = _tmp
                            nextScenar = _t
                        End If
                    End If

                Case Scenario.EDate.OneTime
                    If _t.TheDate.Time = _now Then
                        _t.Go()
                    End If

                    If _t.Enabled AndAlso _t.Done = False Then
                        _tmp = _t.TheDate.Time - (New Scenario.TTime(Date.Now))
                        If _tmp < diff AndAlso _tmp > 0 Then
                            diff = _tmp
                            nextScenar = _t
                        End If
                    End If

                Case Scenario.EDate.Weekly
                    If _t.TheDate.Time.ReadyForWeekly(New Scenario.TTime(Date.Now)) Then
                        _t.Go()
                    End If

                    If _t.Enabled AndAlso _t.Done = False Then
                        _tmp = _t.TheDate.Time.WeeklyDiff(New Scenario.TTime(Date.Now))
                        If _tmp < diff AndAlso _tmp > 0 Then
                            diff = _tmp
                            nextScenar = _t
                        End If
                    End If

            End Select

        Next
        Scenarios.SemProtectScenarios.ReleaseMutex()


        ' Change state
        If _scenarioList.GetScenarios.Count > 0 Then
            If diff < 0 OrElse diff = Long.MaxValue Then
                ' Then there is no scenario enabled
                Me.lblState.Text = "No scenario will be launched"
            Else
                ' Displays difference between NOW and the next date of scenario
                Me.lblState.Text = nextScenar.Action.ToString & " in " & Misc.FormatedSeconds(diff)
            End If
        Else
            Me.lblState.Text = "There is no scenario running"
        End If
        Me.NotifyIcon.Text = Me.lblState.Text

    End Sub

    ' Launch an immediate scenario
    Private Sub launchImmediateScenario(ByVal action As Scenario.EAction, _
                                        Optional ByVal force As Boolean = False, _
                                        Optional ByVal remote As Boolean = False, _
                                        Optional ByVal appPath As String = Nothing)

        Dim _dat As New Scenario.TDate
        Dim _remote As New Scenario.TRemote

        _dat = New Scenario.TDate(Scenario.EDate.Now, _
                                  New Scenario.TTime(0, 0, 0, 0, 0, 0))
        _remote = New Scenario.TRemote(Me.txtIMachineName.Text, _
                                       Me.txtIUserName.Text, Me.txtIPwd.SecureText)

        Dim _t As Scenario
        If remote Then
            _t = New Scenario(_dat, force, action, _remote, appPath)
        Else
            _t = New Scenario(_dat, force, action, appPath)
        End If
        _t.Enabled = True
        _t.AutoRemove = False
        _t.Go()
    End Sub

    Private Sub cmdPreferences_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPreferences.Click
        Dim frm As New frmPreferences
        frm.ShowDialog()
    End Sub

    Private Sub MenuItemAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemAbout.Click
        Dim frm As New frmAboutG
        frm.ShowDialog()
    End Sub

    Private Sub MenuItemUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemUpdate.Click
        Program.Updater.CheckUpdates(False)
    End Sub

    Private Sub MenuItemPreferences_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemPreferences.Click
        Dim frm As New frmPreferences
        frm.ShowDialog()
    End Sub

    Private Sub cmdAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdAbout.Click
        Me.MenuItemAbout_Click(Nothing, Nothing)
    End Sub
End Class
