' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports Misc

Public Class frmPreferences

    Private Sub cmdQuit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdQuit.Click
        ' Apply changes
        _frmMain.dt.CustomFormat = My.Settings.DateTimeFormat
        Me.Close()
    End Sub

    Private Sub cmdSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSave.Click
        ' Save
        My.Settings.UpdateAlpha = Me.chkUpdateAlpha.Checked
        My.Settings.UpdateBeta = Me.chkUpdateBeta.Checked
        My.Settings.AutoUpdate = Me.chkUpdateAuto.Checked
        My.Settings.UpdateServer = Me.txtUpdateServer.Text
        My.Settings.StartWithWindows = Me.chkStart.Checked
        My.Settings.StartMinimized = Me.chkStartMinimized.Checked
        My.Settings.StartEnabled = Me.chkStartEnabled.Checked
        My.Settings.DateTimeFormat = Me.txtDtCustomFormat.Text

        ' Apply 'start with windows'
        Misc.StartWithWindows(My.Settings.StartWithWindows)

        Try
            My.Settings.Save()
            MessageBox.Show("Saved preferences sucessfully.", _
                        "Preferences", _
                        MessageBoxButtons.OK, _
                        MessageBoxIcon.Information)
        Catch ex As Exception
            MessageBox.Show("Failed to save preferences." & vbNewLine & ex.ToString, _
                        "Preferences", _
                        MessageBoxButtons.OK, _
                        MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub frmPreferences_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        CloseWithEchapKey(Me)

        Misc.SetToolTip(Me.chkStart, "Start Simple Shutdown Scheduler on Windows startup")
        Misc.SetToolTip(Me.cmdSave, "Save configuration")
        Misc.SetToolTip(Me.cmdQuit, "Quit without saving")
        Misc.SetToolTip(Me.cmdDefaut, "Set default configuration")
        Misc.SetToolTip(Me.chkUpdateAlpha, "Check for alpha releases")
        Misc.SetToolTip(Me.chkUpdateBeta, "Check for beta releases")
        Misc.SetToolTip(Me.chkUpdateAuto, "Check for updates at startup")
        Misc.SetToolTip(Me.cmdUpdateCheckNow, "Check for updates now")
        Misc.SetToolTip(Me.txtUpdateServer, "Update server")
        Misc.SetToolTip(Me.chkStartEnabled, "Start with scenarios enabled")
        Misc.SetToolTip(Me.chkStartMinimized, "Start minimized")
        Misc.SetToolTip(Me.cmdCustomFormatDefault, "Set default custom format")
        Misc.SetToolTip(Me.txtDtCustomFormat, "Preview the custom format of date/time")
        Misc.SetToolTip(Me.dtPreview, "Preview custom format")

        ' Set control's values
        Me.chkUpdateAlpha.Checked = My.Settings.UpdateAlpha
        Me.chkUpdateBeta.Checked = My.Settings.UpdateBeta
        Me.chkUpdateAuto.Checked = My.Settings.AutoUpdate
        Me.txtUpdateServer.Text = My.Settings.UpdateServer
        Me.chkStart.Checked = My.Settings.StartWithWindows
        Me.chkStartMinimized.Checked = My.Settings.StartMinimized
        Me.chkStartEnabled.Checked = My.Settings.StartEnabled
        Me.txtDtCustomFormat.Text = My.Settings.DateTimeFormat

    End Sub

    Private Sub cmdDefaut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdDefaut.Click
        ' Defaut settings
        Me.chkUpdateAlpha.Checked = False
        Me.chkUpdateBeta.Checked = False
        Me.chkUpdateAuto.Checked = False
        Me.txtUpdateServer.Text = "http://sshutdown.sourceforge.net/update.xml"
        Me.chkStart.Checked = False
        Me.chkStartMinimized.Checked = False
        Me.chkStartEnabled.Checked = True
        Me.txtDtCustomFormat.Text = "ddd, dd MMM yyyy HH':'mm':'ss"
    End Sub

    Private Sub cmdUpdateCheckNow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdUpdateCheckNow.Click
        ' Check for updates manually
        ' No silent mode, so it will cause a messagebox to be displayed
        My.Settings.UpdateAlpha = Me.chkUpdateAlpha.Checked
        My.Settings.UpdateBeta = Me.chkUpdateBeta.Checked
        Program.Updater.CheckUpdates(False)
    End Sub

    Private Sub txtDtCustomFormat_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtDtCustomFormat.TextChanged
        Me.dtPreview.CustomFormat = Me.txtDtCustomFormat.Text
    End Sub

    Private Sub cmdCustomFormatDefault_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCustomFormatDefault.Click
        Me.txtDtCustomFormat.Text = "ddd, dd MMM yyyy HH':'mm':'ss"
    End Sub
End Class