﻿' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports System.Runtime.InteropServices

Public Module Program

    ' Public variables
    Public _frmMain As frmMain
    Public _scenarioList As New Scenarios
    Public XML_PATH As String = My.Computer.FileSystem.SpecialDirectories.CurrentUserApplicationData
    Public XML_FILE As String = XML_PATH & "\Scenarios.xml"

    ' Private parameters
    Private _time As Integer
    Private WithEvents _updater As cUpdate

    ' Public properties
    Public ReadOnly Property ElapsedTime() As Integer
        Get
            Return _time
        End Get
    End Property
    Public ReadOnly Property Updater() As cUpdate
        Get
            Return _updater
        End Get
    End Property


    ' Entry point of executable
    Sub Main()


        ' ======= Save time of start
        _time = Native.Api.Win32.GetElapsedTime



        ' ======= Close application if there is a previous instance of SSS running
        If cEnvironment.IsAlreadyRunning Then
            Exit Sub
        End If



        ' ======= Some basic initialisations
        Application.EnableVisualStyles()
        Application.SetCompatibleTextRenderingDefault(False)    ' Use GDI, not GDI+



        ' ======= Enable some privileges
        cEnvironment.RequestPrivilege(cEnvironment.PrivilegeToRequest.ShutdownPrivilege)



        ' ======= Set handler for exceptions
        AddHandler Application.ThreadException, AddressOf MYThreadHandler
        AddHandler AppDomain.CurrentDomain.UnhandledException, AddressOf MYExnHandler
        Application.SetUnhandledExceptionMode(UnhandledExceptionMode.CatchException)



        ' ======= Instanciate all classes
        _frmMain = New frmMain              ' Main form
        _updater = New cUpdate              ' Updater class



        ' ======= Create directory for XML file
        If System.IO.Directory.Exists(XML_PATH) = False Then
            Try
                Call System.IO.Directory.CreateDirectory(XML_PATH)
            Catch ex As Exception
                MessageBox.Show("Cannot create directory " & XML_PATH, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Call ExitSSS()
            End Try
        End If



        ' ======= Load preferences
        Try
            ' Try to update settings from a previous version of SSS
            My.Settings.Upgrade()
        Catch ex As Exception
            '
        End Try
        Try
            If My.Settings.FirstTime Then
                MessageBox.Show("This is the first time you start Simple Shutdown Scheduler." & _
                       Environment.NewLine & "Please visit http://sshutdown.sourceforge.net/ for details !" & _
                       Environment.NewLine & "Please send any feed back or comment to violent_ken [@] users [dot] sourceforge [dot] net." & _
                       Environment.NewLine & Environment.NewLine & "This message will not be displayed anymore.", _
                       "Please read this", MessageBoxButtons.OK, MessageBoxIcon.Information)
                My.Settings.FirstTime = False
                My.Settings.Save()
            End If
        Catch ex As Exception
            ' Preference file corrupted/missing
            MessageBox.Show("Preference file is missing or corrupted and will be now recreated.", "Startup error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            'My.Settings.Reset()
            'My.Settings.Save()
        End Try


        ' ======= Show main form & start application
        Application.Run(_frmMain)

    End Sub

    ' Handler for exceptions
    Private Sub MYExnHandler(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
        Dim ex As Exception
        ex = CType(e.ExceptionObject, Exception)
        Console.WriteLine(ex.StackTrace)
        Dim t As New frmError(ex)
#If RELEASE_MODE = 1 Then
        t.ShowDialog()
#End If
    End Sub
    Private Sub MYThreadHandler(ByVal sender As Object, ByVal e As Threading.ThreadExceptionEventArgs)
        Console.WriteLine(e.Exception.StackTrace)
        Dim t As New frmError(e.Exception)
#If RELEASE_MODE = 1 Then
        t.ShowDialog()
#End If
    End Sub




    ' Exit application
    Public Sub ExitSSS()

        ' Save scenarios
        _scenarioList.SaveData(XML_FILE)

        ' Save settings
        My.Settings.Save()

        ' Close forms & exit
        If _frmMain IsNot Nothing Then
            _frmMain.AbleToClose = True
            _frmMain.Close()
        End If
        Application.Exit()

    End Sub

    ' Generate an unhandled error
    Public Sub ThrowUnhandledError()
        Dim x As IntPtr = IntPtr.Zero
        Runtime.InteropServices.Marshal.ReadInt64(x, 0)
    End Sub

    Private Sub _updater_FailedToCheckVersion(ByVal silent As Boolean, ByVal msg As String) Handles _updater.FailedToCheckVersion
        ' Failed to check update
        If silent Then
            ' Silent mode -> only displays a tooltip
            If _frmMain IsNot Nothing AndAlso _frmMain.NotifyIcon IsNot Nothing Then
                With _frmMain.NotifyIcon
                    .BalloonTipText = msg
                    .BalloonTipIcon = ToolTipIcon.Info
                    .BalloonTipTitle = "Could not check if SSS us ip to date."
                    .ShowBalloonTip(3000)
                End With
            End If
        Else
            _frmMain.Invoke(New frmMain.FailedToCheckUpDateNotification(AddressOf impFailedToCheckUpDateNotification), msg)
        End If
    End Sub

    Private Sub _updater_NewVersionAvailable(ByVal silent As Boolean, ByVal release As cUpdate.NewReleaseInfos) Handles _updater.NewVersionAvailable
        ' A new version of SSS is available
        If silent Then
            ' Silent mode -> only displays a tooltip
            If _frmMain IsNot Nothing AndAlso _frmMain.NotifyIcon IsNot Nothing Then
                With _frmMain.NotifyIcon
                    .BalloonTipText = release.Infos
                    .BalloonTipIcon = ToolTipIcon.Info
                    .BalloonTipTitle = "A new version of SSS is available !"
                    .ShowBalloonTip(3000)
                End With
            End If
        Else
            _frmMain.Invoke(New frmMain.NewUpdateAvailableNotification(AddressOf impNewUpdateAvailableNotification), release)
        End If
    End Sub

    Private Sub _updater_ProgramUpToDate(ByVal silent As Boolean) Handles _updater.ProgramUpToDate
        ' SSS is up to date (no new version available)
        If silent Then
            ' Silent mode -> only displays a tooltip
            If _frmMain IsNot Nothing AndAlso _frmMain.NotifyIcon IsNot Nothing Then
                With _frmMain.NotifyIcon
                    .BalloonTipText = "SSS is up to date !"
                    .BalloonTipIcon = ToolTipIcon.Info
                    .BalloonTipTitle = "No new version of SSS is available."
                    .ShowBalloonTip(3000)
                End With
            End If
        Else
            _frmMain.Invoke(New frmMain.NoNewUpdateAvailableNotification(AddressOf impNoNewUpdateAvailableNotification))
        End If
    End Sub

    ' Called when a new update is available
    ' It's here cause of thread safety
    Public Sub impNewUpdateAvailableNotification(ByVal release As cUpdate.NewReleaseInfos)
        Dim frm As New frmNewVersionAvailable(release)
        frm.ShowDialog()
    End Sub

    ' Called when no new update is available
    ' It's here cause of thread safety
    Public Sub impNoNewUpdateAvailableNotification()
        MessageBox.Show("SSS is up to date !" & vbNewLine & "The current version (" & My.Application.Info.Version.ToString & ") is the latest available for download.", _
                        "SSS update", _
                        MessageBoxButtons.OK, _
                        MessageBoxIcon.Information)
    End Sub

    ' Called when failed to check is SSS is up to date
    Public Sub impFailedToCheckUpDateNotification(ByVal msg As String)
        MessageBox.Show("Could not check if SSS is up to date." & vbNewLine & msg, _
                        "SSS update", _
                        MessageBoxButtons.OK, _
                        MessageBoxIcon.Error)
    End Sub
End Module
